
import csv



class MapRender():
	
	def __init__(self):
		self.context = None
		self.mapData = None
		self.style = None
		self.rect = None
		self.bbox = [-90.,-180.,90.,180.]
		self.zoom = 12
		self.ignoreMissingNodes = 1
	
		#Read CSS Colors from csv file
		self.cssColors = {}
		colData = csv.reader(open("csscolors.csv","rt"))
		for col in colData:
			if len(col) is 0: continue
			self.cssColors[col[0]] = col[1]
		
	def CssColourToRgb(self,col):
		colHex = None
		if col in self.cssColors:
			colHex = self.cssColors[col]
		if col[0] == "#":
			colHex = col
		#Css colours can be used case insensitively
		if colHex is None:
			for cssCol in self.cssColors.keys():
				#print cssCol.lower(),col.lower()
				if cssCol.lower() == col.lower():
					colHex = self.cssColors[cssCol]
				if colHex is not None:
					break

		if colHex is None: raise Exception("Unknown colour "+col)

		r = int(colHex[1:3],16) / 255.
		g = int(colHex[3:5],16) / 255.
		b = int(colHex[5:7],16) / 255.
		#print colHex,r,g,b

		return (r,g,b)

	def Project(self,lat,lon):
		#print lat,lon,self.rect,self.bbox
		fracx = (lon - self.bbox[1]) / (self.bbox[3] - self.bbox[1])
		fracy = (lat - self.bbox[0]) / (self.bbox[2] - self.bbox[0])

		winx = fracx * (self.rect[2]) #+ self.rect[0]
		winy = - fracy * (self.rect[3]) + self.rect[3]
		return (winx,winy)

	def UnProject(self,winx,winy):
		fracx = (winx - self.rect[0]) / (self.rect[2])
		fracy = (self.rect[3] - winy) / (self.rect[3])

		lon = fracx * (self.bbox[3] - self.bbox[1]) + self.bbox[1]
		lat = fracy * (self.bbox[2] - self.bbox[0]) + self.bbox[0]  

		return (lat,lon)

	def CalcCentreOnDrag(self,mouseWinPos,worldPos):
		#print mouseWinPos,worldPos

		xRange = self.bbox[3] - self.bbox[1]
		yRange = self.bbox[2] - self.bbox[0]
		xWin = (self.rect[2])
		yWin = (self.rect[3])	
	
		temp =[None,None,None,None]

		temp[1] = worldPos[1] - xRange * (mouseWinPos[0] - self.rect[0]) / xWin
		temp[0] = worldPos[0] - yRange * (self.rect[3]- mouseWinPos[1]) / yWin

		temp[2] = temp[0] + yRange
		temp[3] = temp[1] + xRange

		return (temp[0] + 0.5 * yRange, temp[1] + 0.5 * xRange)

	def RenderWay(self,typ,ref):
		shape = self.mapData.GetShape(typ,ref)
		tags = self.mapData.GetTags(typ,ref)

		for seg in shape:
			
			count = 0
			area = seg['closed']

			#Get Style for shape segment
			properties = {}
			properties['type'] = typ
			properties['area'] = area
			properties['canvas'] = False
			properties['line'] = (typ == "way" and not area)
			objStyle = self.style.GetStyle(tags,self.zoom,properties)
			#print objStyle

			self.context.save()
			for point in seg['shape']:
				if self.ignoreMissingNodes:
					if point is None: continue				

				pwin = self.Project(*point)
				if count == 0:
					self.context.move_to(*pwin)
				else:
					self.context.line_to(*pwin)

				count = count + 1

			if area and "fill-color" in objStyle:
				fillAlpha = 1.
				if "fill-opacity" in objStyle: fillAlpha = float(objStyle['fill-opacity'])
				
				self.context.close_path()
				col = self.CssColourToRgb(objStyle['fill-color'])
				self.context.set_source_rgba(col[0],col[1],col[2],fillAlpha)
				self.context.fill_preserve()

			if "color" in objStyle:
				lineWidth = 1.
				if "width" in objStyle: lineWidth = float(objStyle['width'])
				lineAlpha = 1.
				if "opacity" in objStyle: lineAlpha = float(objStyle['opacity'])

				self.context.set_line_width(lineWidth)
				col = self.CssColourToRgb(objStyle['color'])
				self.context.set_source_rgba(col[0],col[1],col[2],lineAlpha)
				self.context.stroke()
	            	self.context.restore()

	def Render(self,context):
		print self.rect, self.bbox
		self.context = context
		for obj in self.mapData.GetTaggedObjs():
			self.RenderWay(obj['type'],obj['ref'])



