
import cssutils, re

#http://stackoverflow.com/questions/241327/python-snippet-to-remove-c-and-c-comments/241506#241506
def CommentRemover(text):
    def replacer(match):
        s = match.group(0)
        if s.startswith('/'):
            return ""
        else:
            return s
    pattern = re.compile(
        r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"',
        re.DOTALL | re.MULTILINE
    )
    return re.sub(pattern, replacer, text)


class MapStyle():
	def __init__(self, filename):

		#Remove line breaks
		s = ""
		for line in open(filename,"rt").readlines():
			s = s + line.strip()

		#Remove comments
		nocom = CommentRemover(s)

		#Split rules
		closeBrak = nocom.split("}")
		rulesRaw = []
		for rule in closeBrak:
			result = rule.split("{")
			if len(result) is 1: continue
			rulesRaw.append(result)

		#Split declarations
		rulesDec = []
		for rule in rulesRaw:
			declarations = rule[1].split(";")
			decFiltered = []
			for p in declarations:
				content = p.strip()
				if len(content) > 0:
					decFiltered.append(content)

			rulesDec.append((rule[0], decFiltered))

		#Split properties from values
		rulesProp = []
		for rule in rulesDec:
			decs = rule[1]
			decsFilt = []
			for dec in decs:
				prva = dec.split(":")
				prpty = prva[0].strip()
				val = prva[1].strip()
				decsFilt.append((prpty,val))


			rulesProp.append((rule[0],decsFilt))

		#Split selectors
		rulesSel = []
		for rule in rulesProp:
			selectors = rule[0].split(",")
			selFilt = []
			for s in selectors:
				selFilt.append(s.strip())
			rulesSel.append((selFilt, rule[1]))

		#Split selector into parts
		rulesSelParts = []
		for rule in rulesSel:
			selFilt = []
			for sel in rule[0]:
				
				decFilt = []
				decend = sel.split()
				for d in decend:
					tests = d.split("[")
					testsFilt = [[]]
					for basic in tests[0].split("|"):
						testsFilt[0].append(basic.strip())
					for t in tests[1:]:
						testsFilt.append(t.rstrip("] "))
					decFilt.append(testsFilt)
					#decFilt.append(d)
				selFilt.append(decFilt)
			rulesSelParts.append((selFilt,rule[1]))

		#Print result
		if 0:
			for rule in rulesSelParts:
				for sel in rule[0]:
					print sel
				for dec in rule[1]:
					print "\t",dec

		self.rules = rulesSelParts

	def CheckRule(self,rule, tags, zoom, properties):
		#print rule
		selectorList = rule[0]
		selectorListMatched = 0
		for descendants in selectorList:
			if selectorListMatched: continue #No point carrying on when it already matches

			#print descendants
			if len(descendants) > 1:
				raise Exception("Descendants combinator not supported (yet)")

			selector = descendants[-1] #Just use last check in descendants combinator (for now)
			selectorMatched = None
			basicSel = selector[0]
			tests = selector[1:]
			basicType = basicSel[0]
			zoomTest = None
			if len(basicSel) > 1:
				zoomTest = basicSel[1]
			#print basicType, zoomTest, tests

			#Check basic type
			typ = properties['type']
			basicTypeMatched = None
			if basicType in ['node','way','relation']:
				basicTypeMatched = (typ == basicType)
			if basicType == "area":
				basicTypeMatched = properties["area"]
			if basicType == "line":
				basicTypeMatched = properties["line"]
			if basicType == "canvas":
				basicTypeMatched = properties["canvas"]
			if basicTypeMatched:
				selectorMatched = 1

			#Check zoom
			if selectorMatched and zoomTest is not None:
				if zoomTest[0] != "z":
					raise Exception("First character of zoom test should be 'z'")
				zoomIsRange = (zoomTest.find("-") != -1)
				if zoomIsRange:
					zoomRang = zoomTest[1:].split("-")
					minZoom = zoomRang[0].strip()
					maxZoom = zoomRang[1].strip()
					if len(minZoom)>0: minZoom = int(minZoom)
					else: minZoom = None
					if len(maxZoom)>0: minZoom = int(maxZoom)
					else: maxZoom = None
				else:
					z = int(zoomTest[1:])
					minZoom = z
					maxZoom = z
				#print "zr",minZoom, maxZoom
				if minZoom is not None and zoom < minZoom:
					selectorMatched = 0
				if maxZoom is not None and zoom > maxZoom:
					selectorMatched = 0

			

			#Do tag tests
			for test in tests:
				if not selectorMatched: continue
				greaterOperator = (test.find(">=") != -1)
				lesserOperator = (test.find("<=") != -1)
				if greaterOperator or lesserOperator:
					raise Exception ("Math operators not supported (yet)")

				equalityTest = (test.find("=") != -1)
				if equalityTest:
					(key,val) = test.split("=")
					if key not in tags:
						selectorMatched = 0
						continue
					if val != tags[key]:
						selectorMatched = 0
						continue						
				else:
					key = test
					notOperator = (test[0] == "!")
					if notOperator:
						key = test[1:]
					#print "key",key
					if key not in tags and not notOperator:
						selectorMatched = 0
						continue
					if key in tags and notOperator:
						selectorMatched = 0
						continue

			if selectorMatched:
				selectorListMatched = 1
		#if selectorListMatched:
		#	print rule

		return selectorListMatched

	def ApplyRuleToObj(self,rule, objStyle):
		decs = rule[1]
		for d in decs:
			objStyle[d[0]] = d[1]

	def GetStyle(self, tags, zoom, properties):
		objStyle = {}

		#print "Getting style for",tags, properties
		for rule in self.rules:
			#print rule
			match = self.CheckRule(rule, tags, zoom, properties)

			if match:
				self.ApplyRuleToObj(rule, objStyle)

		#print objStyle
		return objStyle

