#!/usr/bin/env python

import gtk
from gtk import gdk
import gobject

import math
from datetime import datetime
from mapRender import MapRender

class MapWidget(gtk.DrawingArea):
    # signals
    __gsignals__ = {'view_changed':(gobject.SIGNAL_RUN_LAST,
                                      gobject.TYPE_NONE,
                                      ())}

    def __init__(self):
	gobject.GObject.__init__(self)
        super(MapWidget, self).__init__()

        # gtk.Widget signals
        self.connect("expose_event", self.expose)
        self.connect("button_press_event", self.button_press)
        self.connect("button_release_event", self.button_release)
        self.connect("motion_notify_event", self.motion_notify)
	self.connect("scroll-event", self.MouseWheel)

        # unmask events
        self.add_events(gdk.BUTTON_PRESS_MASK |
                        gdk.BUTTON_RELEASE_MASK |
                        gdk.POINTER_MOTION_MASK |
			gdk.SCROLL_MASK)

	self.set_size_request(40,40)
	

        # update the clock once a second
        gobject.timeout_add(1000, self.update)

	self.mapRender = MapRender()
	self.mapData = None
	self.mapStyle = None
	self.centrePos = (51.11,0.92)
	self.zoom = 12
	self.rightClickDrag = 0
	self.rightClickDragStart = None
	self.rightClickDragWorldPos = None

    def expose(self, widget, event):
        context = widget.window.cairo_create()

        # set a clip region for the expose event
        context.rectangle(event.area.x, event.area.y,
                          event.area.width, event.area.height)
        context.clip()

        self.draw(context)

        return False

    def MouseWheel(self, widget, event):
	#print event
	if event.direction.value_name == "GDK_SCROLL_UP":
		self.zoom = self.zoom + 1
	if event.direction.value_name == "GDK_SCROLL_DOWN":
		self.zoom = self.zoom - 1

	self.redraw_canvas()
	self.emit("view_changed")

        return False

    def button_press(self, widget, event):

	if event.button is 3:
		self.rightClickDrag = True
		self.rightClickDragStart = (event.x,event.y)
		self.rightClickDragWorldPos = self.mapRender.UnProject(event.x,event.y)
		#print self.rightClickDragWorldPos
		#self.emit("view_changed", self.centrePos, self.zoom)

        return False

    def SetView(self,centre,zoom):
	self.centrePos = centre
	self.zoom = zoom
	self.redraw_canvas()

    def button_release(self, widget, event):
	#print event
	if event.button is 3:
		self.rightClickDrag = False
		self.rightClickDragStart = None
		self.rightClickDragWorldPos = None
		self.redraw_canvas()
		self.emit("view_changed")

        return False

    def motion_notify(self, widget, event):
	#print event
        if self.rightClickDrag:
		#print event,self.rightClickDragStart
		self.centrePos = self.mapRender.CalcCentreOnDrag((event.x,event.y),self.rightClickDragWorldPos)

    def draw(self, context):
        rect = self.get_allocation()
	widgetWidth = (rect[2])
	widgetHeight = (rect[3])

	#bbox = [51.0912248,0.8853291,51.1365207,0.9674404]
	latHalfWidth = widgetHeight * 0.000075 / pow(2,self.zoom-12)
	lonHalfWidth = widgetWidth * 0.000075 / pow(2,self.zoom-12)
	#print self.centrePos
	bbox = [self.centrePos[0]-latHalfWidth,self.centrePos[1]-lonHalfWidth,
		self.centrePos[0]+latHalfWidth,self.centrePos[1]+lonHalfWidth]

	self.mapRender.context = context
	self.mapRender.mapData = self.mapData
	self.mapRender.style = self.mapStyle
	self.mapRender.rect = rect
	self.mapRender.bbox = bbox
	self.mapRender.zoom = self.zoom
	self.mapRender.Render(context)

    def redraw_canvas(self):
        if self.window:
            alloc = self.get_allocation()
            #rect = gdk.Rectangle(alloc.x, alloc.y, alloc.width, alloc.height)
            #self.window.invalidate_rect(rect, True)
            #self.queue_draw_area(alloc.x, alloc.y, alloc.width, alloc.height)
	    self.queue_draw_area(0, 0, alloc.width, alloc.height)
            self.window.process_updates(True)

    def update(self):
        return True # keep running this event


#gobject.type_register(MapWidget)
#gobject.signal_new('view_changed', MapWidget, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []) 


def main():
    window = gtk.Window()
    mapWidget = MapWidget()

    window.add(mapWidget)
    window.connect("destroy", gtk.main_quit)
    window.show_all()

    gtk.main()

if __name__ == "__main__":
    main()

