
import os.path
import gzip
from xml.parsers import expat

class OsmDataFileStream():

	def __init__ (self):
		self.textBuff = ""
		self.data = {}
		self.current = None
		self.currentType = ""
		self.depth = 0
		for t in ["node","way","relation"]:
			self.data[t] = {}

	def CharacterData(self, data):
		data = data.strip()
		if data:
			data = data.encode('utf-8')
			self.textBuff += data + "\n"

	def StartElement(self,name, attr):
		name = name.encode('utf-8')
		self.textBuff = ""
		attrUtf8 = {}
		for attrk in attr.keys():
			attrUtf8[attrk.encode('utf-8')] = attr[attrk].encode('utf-8')
		if "id" in attrUtf8.keys():
			attrUtf8["id"] = int(attrUtf8["id"])
		
		if self.depth is 1 and name in ["node","way","relation"]:
			self.current = {}
			self.currentType = name
			for t in ['attr','tags']:
				self.current[t] = {}
			self.current['mem'] = []
			if "lat" in attrUtf8.keys():
				attrUtf8["lat"] = float(attrUtf8["lat"])
			if "lon" in attrUtf8.keys():
				attrUtf8["lon"] = float(attrUtf8["lon"])
			self.current['attr'] = attrUtf8
			#print self.depth, name, attr

		if self.depth is 2 and self.current is not None and name == "tag":
			key = attrUtf8['k']
			val = attrUtf8['v']
			self.current['tags'][key] = val

		if self.depth is 2 and self.current is not None and name == "nd":
			ref = int(attrUtf8['ref'])
			self.current['mem'].append({"type":"node","ref":ref,"role":None})

		if self.depth is 2 and self.current is not None and name == "member":
			ref = int(attrUtf8['ref'])
			typ = attrUtf8['type']
			role = attrUtf8['role']
			self.current['mem'].append({"type":typ,"ref":ref,"role":role})

		self.depth = self.depth + 1

	def EndElement(self,name):
		name = name.encode('utf-8')
		self.textBuff = ""
		self.depth = self.depth - 1

		#Store current done in local dict
		if self.depth is 1 and self.current is not None:
			objId = int(self.current['attr']['id'])
			#print self.currentType, objId
			self.data[self.currentType][objId] = self.current
			self.current = None
		

	def Read(self, fi):
		xmlParser = expat.ParserCreate()
		xmlParser.CharacterDataHandler = self.CharacterData
		xmlParser.StartElementHandler = self.StartElement
		xmlParser.EndElementHandler = self.EndElement

		#fileSize = os.path.getsize(fName)
		count = 0

		while 1:
			data = fi.read(1024)
			count = count + len(data)
			if data == "":
				break
			xmlParser.Parse(data, 0)
			#progress = float(count)/float(fileSize)

		xmlParser.Parse("", 1)
		


class OsmDataFile():
	def __init__(self, filename):
		self.filename = filename
		self.fi = None

		ext = self.filename.split(".")
		#print ext[-1], ext[-2]
		if len(ext) >= 3 and ext[-1] == "gz" and ext[-2] == "osm":
			self.fi = gzip.open(self.filename,"rt")
		if len(ext) >= 2 and ext[-1] == "osm":
			self.fi = open(self.filename,"rt")
		
		if self.fi is None:
			raise Exception("Unrecognised file extension")

		
		stream = OsmDataFileStream()
		stream.Read(self.fi)

		self.data = stream.data

		#print self.data['relation']
		#for rec in self.GetTaggedObjs():
		#	obj = self.data[rec['type']][rec['ref']]
		#	if 'building' not in obj['tags']: continue
		#	print rec
		#	print self.GetShape(rec['type'],rec['ref'])

	def GetTaggedObjs(self):
		out = []

		for ty in self.data.keys():
			for i in self.data[ty].keys():
				obj = self.data[ty][i]
				if len(obj['tags']) is 0: continue
				out.append({"type":ty,"ref":obj['attr']['id']})
		return out

	def GetTags(self,typ,ref):
		if typ not in self.data: return None
		if ref not in self.data[typ]: return None
		return self.data[typ][ref]['tags']

	def GetPosition(self,typ,ref):
		if typ not in self.data: return None
		if ref not in self.data[typ]: return None
		obj = self.data[typ][ref]

		if 'lat' in obj['attr'] and 'lon' in obj['attr']:
			return (obj['attr']['lat'], obj['attr']['lon'])

		return None

	def GetShape(self,typ,ref):
		if typ not in self.data: return None
		if ref not in self.data[typ]: return None
		obj = self.data[typ][ref]
		
		currentSeg = {'shape': [], 'closed': False, 'point': False}

		if 'lat' in obj['attr'] and 'lon' in obj['attr']:
			currentSeg['shape'].append((obj['attr']['lat'], obj['attr']['lon']))
			currentSeg['point'] = True

		for mem in obj['mem']:
			if mem['type'] != "node": continue #Only consider nodes in ways for now
			currentSeg['shape'].append(self.GetPosition(mem['type'],mem['ref']))

		if len(obj['mem']) > 0:
			firstMem = obj['mem'][0]
			lastMem = obj['mem'][-1]
			if firstMem['type'] == lastMem['type'] and firstMem['type'] == "node" and firstMem['ref'] == lastMem['ref']:
				currentSeg['closed'] = True

		return [currentSeg]


